@@ -15,11 +15,16 @@ def user_mean(func, true, pred, k):
         return np.mean(p.map(func, [data for user, data in y.items()]))
 
 
-def unique_items(pred, k):
+def user_apply(func, df, pred, k, fill):
+    pred = top_k(pred, k)
     with Pool(cpu_count()) as p:
-        topk = p.starmap(get_k, [(data, k) for user, data in pred.items()])
-        topk = [item for sublist in topk for item in sublist]
-        return np.unique(topk)
+        return np.mean(p.starmap(func, [(df, data, fill) for user, data in pred.items()]))
+
+
+def top_k(pred, k):
+    with Pool(cpu_count()) as p:
+        top_items = p.starmap(get_k, [(data, k) for user, data in pred.items()])
+        return {user: top for user, top in zip(pred, top_items)}
 
 
 def get_k(l, k):
